package com.muyu.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.benmanes.caffeine.cache.Cache;
import com.muyu.config.tenant.CustomTenantHandler;
import com.muyu.system.constants.SecurityConstants;
import com.muyu.system.domain.LoginUserInfo;
import com.muyu.system.handle.SystemHandler;
import com.muyu.system.properties.ServerConfigProperties;
import com.muyu.utils.IdUtils;
import com.muyu.utils.JwtUtils;
import com.muyu.web.domain.ServerConfig;
import com.muyu.web.domain.UserInfo;
import com.muyu.web.domain.req.UserRegReq;
import com.muyu.web.mapper.UserInfoMapper;
import com.muyu.web.service.ServerConfigService;
import com.muyu.web.service.SystemAuthService;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Author: DongZeLiang
 * @date: 2024/9/12
 * @Description: 系统认证业务层
 * @Version: 1.0
 */
@Service
public class SystemAuthServiceImpl implements SystemAuthService {

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private ServerConfigService serverConfigService;

    @Resource( name = "loginUserCache")
    private Cache<String, LoginUserInfo> loginUserInfoCache;
    @Autowired
    private CustomTenantHandler customTenantHandler;

    /**
     * 用户登录
     *
     * @param userName 用户名称
     * @param password 用户密码
     *
     * @return 生成令牌
     */
    @Override
    public String login (String userName, String password) {
        LambdaQueryWrapper<UserInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserInfo::getUserName, userName);
        queryWrapper.eq(UserInfo::getPassword, password);
        List<UserInfo> userInfoList = userInfoMapper.selectList(queryWrapper);
        if (userInfoList.isEmpty()){
            throw new RuntimeException("用户用户名或者密码错误");
        }
        UserInfo userInfo = userInfoList.get(0);
        LambdaQueryWrapper<ServerConfig> serverConfigQueryWrapper = new LambdaQueryWrapper<>();
        serverConfigQueryWrapper.eq(ServerConfig::getTenantId, userInfo.getTenantId());
        ServerConfig serverConfig = serverConfigService.getOne(serverConfigQueryWrapper);
        LoginUserInfo loginUserInfo = LoginUserInfo.builder()
                .id(userInfo.getId())
                .tenantId(userInfo.getTenantId())
                .userName(userInfo.getUserName())
                .serverConfig(
                        ServerConfigProperties.configToProperties(serverConfig)
                )
                .build();

        String userKey = IdUtils.fastUUID();
        loginUserInfo.setUserKey(userKey);
        loginUserInfoCache.put(userKey, loginUserInfo);
        // Jwt存储信息
        Map<String, Object> claimsMap = new HashMap<String, Object>();
        claimsMap.put(SecurityConstants.USER_KEY, userKey);
        claimsMap.put(SecurityConstants.DETAILS_USER_ID, userInfo.getId());
        claimsMap.put(SecurityConstants.DETAILS_USERNAME, userName);

        return  JwtUtils.createToken(claimsMap);
    }

    /**
     * 用户登出
     */
    @Override
    public void logout () {
        loginUserInfoCache.invalidate(SystemHandler.getUserKey());
    }

    /**
     * 获取登录用户信息
     *
     * @return 登录用户信息
     */
    @Override
    public LoginUserInfo info () {
        return SystemHandler.getUserInfo();
    }

    /**
     * 用户注册
     *
     * @param userRegReq 用户注册请求
     */
    @Override
    @Transactional
    public void reg (UserRegReq userRegReq) {
        LambdaQueryWrapper<UserInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(UserInfo::getUserName, userRegReq.getUserName());
        Long selectCount = userInfoMapper.selectCount(queryWrapper);
        if (!selectCount.equals(0L)){
            throw new RuntimeException("用户名称重复，请重新输入");
        }
        UserInfo userInfo = UserInfo.builder()
                .userName(userRegReq.getUserName())
                .password(userRegReq.getPassword())
                .createTime(new Date())
                .tenantId("TX_" + IdUtils.simpleUUID().substring(0, 10).toUpperCase())
                .build();
        this.userInfoMapper.insert(userInfo);
        customTenantHandler.ignore();
        serverConfigService.save(
                ServerConfig.builder()
                        .tenantId(userInfo.getTenantId())
                        .host("127.0.0.1")
                        .port("81")
                        .uri("/verify/vehicleConnection")
                        .defaultMqttAddr("127.0.0.1")
                        .defaultMqttTopic("vehicle")
                        .defaultMqttQos(1)
                        .build()
        );
        customTenantHandler.remove();

    }
}
